@@ -169,9 +169,9 @@ eel_background_class_init (gpointer klass)
 			      G_STRUCT_OFFSET (EelBackgroundClass,
 					       settings_changed),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
+			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE,
-			      0);
+			      1, G_TYPE_INT);
 	signals[DETERMINE_IMAGE_PLACEMENT] = 
 		g_signal_new ("determine_image_placement",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -304,7 +304,7 @@ eel_background_set_image_placement (EelBackground              *background,
 {
 	if (eel_background_set_image_placement_no_emit (background, new_placement)) {
 		g_signal_emit (G_OBJECT (background),
-			       signals[SETTINGS_CHANGED], 0);
+			       signals[SETTINGS_CHANGED], 0, GDK_ACTION_COPY);
 		g_signal_emit (G_OBJECT (background),
 			       signals[APPEARANCE_CHANGED], 0);
 	}
@@ -1215,7 +1215,7 @@ eel_background_set_color (EelBackground *background,
 			  const char *color)
 {
 	if (eel_background_set_color_no_emit (background, color)) {
-		g_signal_emit (G_OBJECT (background), signals[SETTINGS_CHANGED], 0);
+		g_signal_emit (G_OBJECT (background), signals[SETTINGS_CHANGED], 0, GDK_ACTION_COPY);
 		if (!eel_background_image_totally_obscures (background)) {
 			g_signal_emit (GTK_OBJECT (background), signals[APPEARANCE_CHANGED], 0);
 		}
@@ -1335,7 +1335,7 @@ eel_background_set_image_uri_helper (EelBackground *background,
 	eel_background_start_loading_image (background, emit_appearance_change, load_async);
 	
 	if (emit_setting_change) {
-		g_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED], 0);
+		g_signal_emit (GTK_OBJECT (background), signals[SETTINGS_CHANGED], 0, GDK_ACTION_COPY);
 	}
 
 	return TRUE;
@@ -1353,25 +1353,34 @@ eel_background_set_image_uri_sync (EelBackground *background, const char *image_
 	eel_background_set_image_uri_helper (background, image_uri, TRUE, TRUE, FALSE);
 }
 
+typedef struct 
+{
+	char *color;
+	GdkDragAction action;
+} ImageLoadingData;
+
 static void
-set_image_and_color_image_loading_done_callback (EelBackground *background, gboolean successful_load, char *color)
+set_image_and_color_image_loading_done_callback (EelBackground *background, gboolean successful_load, ImageLoadingData *data)
 {
 	g_signal_handlers_disconnect_by_func
 		(G_OBJECT (background),
-		 G_CALLBACK (set_image_and_color_image_loading_done_callback), color);
+		 G_CALLBACK (set_image_and_color_image_loading_done_callback), data);
 
-	eel_background_set_color_no_emit (background, color);
+	eel_background_set_color_no_emit (background, data->color);
 
-	g_free (color);
-	
 	/* We always emit , even if the color didn't change, because the image change
 	 * relies on us doing it here.
 	 */
-	g_signal_emit (background, signals[SETTINGS_CHANGED], 0);
+
+	g_signal_emit (background, signals[SETTINGS_CHANGED], 0, data->action);
 
 	if (!background->details->emit_after_load) {
 		g_signal_emit (background, signals[APPEARANCE_CHANGED], 0);
 	}
+
+	g_free (data->color);
+	g_free (data);
+	
 }
 
 /* Use this fn to set both the image and color and avoid flash. The color isn't
@@ -1379,25 +1388,27 @@ set_image_and_color_image_loading_done_callback (EelBackground *background, gboo
  * before then, it will use the old color and image.
  */
 static void
-eel_background_set_image_uri_and_color (EelBackground *background, const char *image_uri, const char *color)
+eel_background_set_image_uri_and_color (EelBackground *background, GdkDragAction action, const char *image_uri, const char *color)
 {
-	char *color_copy;
-
+	ImageLoadingData *data;
+	
 	if (eel_strcmp (background->details->color, color) == 0 &&
 	    eel_strcmp (background->details->image_uri, image_uri) == 0) {
 		return;
 	}
 
-	color_copy = g_strdup (color);
+	data = g_new0 (ImageLoadingData, 1);
+	data->color = g_strdup (color);
+	data->action = action;
 
 	g_signal_connect (background, "image_loading_done",
-			  G_CALLBACK (set_image_and_color_image_loading_done_callback), color_copy);
+			  G_CALLBACK (set_image_and_color_image_loading_done_callback), data);
 			    
 	/* set_image_and_color_image_loading_done_callback must always be called
 	 * because we rely on it to:
 	 *  - disconnect the image_loading_done signal handler
 	 *  - emit SETTINGS_CHANGED & APPEARANCE_CHANGED
-	 *  - free color_copy
+	 *  - free data
 	 *  - prevent the common cold
 	 */
 	     
@@ -1406,12 +1417,13 @@ eel_background_set_image_uri_and_color (EelBackground *background, const char *i
 	 * call set_image_and_color_image_loading_done_callback ourselves.
 	 */
 	if (!eel_background_set_image_uri_helper (background, image_uri, FALSE, FALSE, TRUE)) {
-		set_image_and_color_image_loading_done_callback (background, TRUE, color_copy);
+		set_image_and_color_image_loading_done_callback (background, TRUE, data);
 	}
 }
 
 void
 eel_background_receive_dropped_background_image (EelBackground *background,
+						 GdkDragAction action,
 						 const char *image_uri)
 {
 	/* Currently, we only support tiled images. So we set the placement.
@@ -1420,7 +1432,7 @@ eel_background_receive_dropped_background_image (EelBackground *background,
 	 */
 	eel_background_set_image_placement_no_emit (background, EEL_BACKGROUND_TILED);
 	
-	eel_background_set_image_uri_and_color (background, image_uri, NULL);
+	eel_background_set_image_uri_and_color (background, action, image_uri, NULL);
 }
 
 /**
@@ -1728,6 +1740,7 @@ eel_background_is_dark (EelBackground *background)
 void
 eel_background_receive_dropped_color (EelBackground *background,
 				      GtkWidget *widget,
+				      GdkDragAction action,
 				      int drop_location_x,
 				      int drop_location_y,
 				      const GtkSelectionData *selection_data)
@@ -1774,7 +1787,7 @@ eel_background_receive_dropped_color (EelBackground *background,
 	
 	g_free (color_spec);
 
-	eel_background_set_image_uri_and_color (background, NULL, new_gradient_spec);
+	eel_background_set_image_uri_and_color (background, action, NULL, new_gradient_spec);
 
 	g_free (new_gradient_spec);
 }
