@@ -27,6 +27,7 @@
 #define EEL_GDK_EXTENSIONS_H
 
 #include <gdk/gdktypes.h>
+#include <gdk/gdkwindow.h>
 
 #define EEL_RGB_COLOR_RED	0xFF0000
 #define EEL_RGB_COLOR_GREEN	0x00FF00
@@ -127,6 +128,7 @@ void                eel_fill_rectangle                     (GdkDrawable
 gboolean            eel_gdk_color_is_dark                  (GdkColor            *color);
 
 /* A routine to get a 50% gray stippled bitmap for use in some types of highlighting. */
+GdkBitmap *         eel_stipple_bitmap_for_screen          (GdkScreen *screen);
 GdkBitmap *         eel_stipple_bitmap                     (void);
 
 
@@ -165,5 +167,11 @@ EelGdkGeometryFlags eel_gdk_parse_geometry                 (const char
 							    int                 *y_return,
 							    guint               *width_return,
 							    guint               *height_return);
-
+void                eel_gdk_draw_layout_with_drop_shadow   (GdkDrawable         *drawable,
+							    GdkGC               *gc,
+							    GdkColor            *text_color,
+							    GdkColor            *shadow_color,
+							    int                  x,
+							    int                  y,
+							    PangoLayout         *layout);
 #endif /* EEL_GDK_EXTENSIONS_H */
