@@ -31,6 +31,7 @@
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-string.h>
+#include <eel/eel-i18n.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkliststore.h>
@@ -38,12 +39,13 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtktreeview.h>
-#include <libgnome/gnome-i18n.h>
+#include <gtk/gtkscrolledwindow.h>
 #include <libgnome/gnome-util.h>
 
 #define NUM_CATEGORY_COLUMNS 1
 #define CATEGORY_COLUMN 0
-#define SPACING_BETWEEN_CATEGORIES_AND_PANES 4
+#define SPACING_BETWEEN_CATEGORIES_AND_PANES 8 
+#define BORDER_WIDTH 5
 #define STRING_LIST_DEFAULT_TOKENS_DELIMETER ","
 
 typedef struct
@@ -342,7 +344,10 @@ GtkWidget*
 eel_preferences_box_new (void)
 {
 	GtkWidget         *tree_view;
+	GtkWidget         *scrolled_window;
 	EelPreferencesBox *preferences_box;
+	GtkWidget         *categories_vbox;
+	GtkWidget         *categories_label;
 
 	preferences_box = EEL_PREFERENCES_BOX
 		(gtk_widget_new (eel_preferences_box_get_type (), NULL));
@@ -350,16 +355,36 @@ eel_preferences_box_new (void)
 	/* Configure ourselves */
  	gtk_box_set_homogeneous (GTK_BOX (preferences_box), FALSE);
  	gtk_box_set_spacing (GTK_BOX (preferences_box), SPACING_BETWEEN_CATEGORIES_AND_PANES);
+	gtk_container_set_border_width (GTK_CONTAINER (preferences_box), BORDER_WIDTH);
 
 	/* The category list */
+	categories_vbox = gtk_vbox_new (FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (preferences_box), 
+			    categories_vbox, FALSE, TRUE, 0);
+
+	categories_label = gtk_label_new_with_mnemonic (_("Cat_egories:"));
+	gtk_misc_set_alignment (GTK_MISC (categories_label), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (categories_vbox), 
+			    categories_label, FALSE, FALSE, 0);
+
 	preferences_box->details->category_store = gtk_list_store_new (
 		NUM_CATEGORY_COLUMNS, G_TYPE_STRING);
 
+	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+					GTK_POLICY_NEVER,
+					GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+					     GTK_SHADOW_IN);
 	tree_view = gtk_tree_view_new_with_model (
 		GTK_TREE_MODEL (preferences_box->details->category_store));
+	gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
 
 	preferences_box->details->category_view = GTK_TREE_VIEW (tree_view);
 
+	gtk_label_set_mnemonic_widget (GTK_LABEL (categories_label),
+				       tree_view);
+
 	gtk_tree_view_insert_column_with_attributes
 		(preferences_box->details->category_view,
 		 0, "", gtk_cell_renderer_text_new (),
@@ -377,7 +402,8 @@ eel_preferences_box_new (void)
 		G_CALLBACK (category_list_selection_changed),
 		preferences_box);
 	
-	gtk_box_pack_start (GTK_BOX (preferences_box), tree_view, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (categories_vbox), 
+			    scrolled_window, TRUE, TRUE, 0);
 
 	/* The gtk notebook that the panes go into. */
 	preferences_box->details->pane_notebook = gtk_notebook_new ();
@@ -391,7 +417,7 @@ eel_preferences_box_new (void)
 			    TRUE,
 			    0);
 
-	gtk_widget_show (tree_view);
+	gtk_widget_show_all (categories_vbox);
 	gtk_widget_show (preferences_box->details->pane_notebook);
 
 	return GTK_WIDGET (preferences_box);
@@ -483,7 +509,9 @@ preferences_box_populate_pane (EelPreferencesBox *preferences_box,
 	EelPreferencesItem *item;
 	EelStringList *group_names;
 	const char *translated_group_name;
+	char **translated_descriptions;
 	guint i;
+	int j;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_BOX (preferences_box));
 	g_return_if_fail (pane_name != NULL);
@@ -521,6 +549,25 @@ preferences_box_populate_pane (EelPreferencesBox *preferences_box,
 										     items[i].preference_name,
 										     items[i].item_type,
 										     items[i].column));
+			if (items[i].accessible_description) {
+				eel_preferences_item_set_accessible_description (item,
+										 gettext (items[i].accessible_description));
+			}
+
+			if (items[i].accessible_description_array) {
+				translated_descriptions = 
+					g_new (char *, items[i].n_accessible_descriptions);
+				
+				for (j = 0; j < items[i].n_accessible_descriptions; j++) {
+					translated_descriptions[j] = gettext (items[i].accessible_description_array[j]);
+				}
+				
+				eel_preferences_item_set_accessible_description_array 
+					(item,
+					 items[i].n_accessible_descriptions,
+					 (const char **)translated_descriptions);
+				g_free (translated_descriptions);
+			}
 			
 			/* Install a control preference if needed */
 			if (items[i].control_preference_name != NULL) {
@@ -719,3 +766,9 @@ eel_preferences_box_get_pane_name (const EelPreferencesBox *preferences_box,
 
 	return NULL;
 }
+
+char *
+eel_preferences_box_get_active_pane (EelPreferencesBox *preferences_box)
+{
+	return g_strdup (preferences_box->details->selected_pane);
+}
