@@ -27,6 +27,7 @@
 #include <config.h>
 #include "eel-preferences-item.h"
 
+#include "eel-accessibility.h"
 #include "eel-art-gtk-extensions.h"
 #include "eel-enumeration.h"
 #include "eel-font-picker.h"
@@ -127,7 +128,7 @@ preferences_item_update_enumeration_radio (EelPreferencesItem *item)
 	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_VERTICAL_RADIO
 			  || item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_HORIZONTAL_RADIO);
 
- 	value = eel_preferences_get_integer (item->details->preference_name);
+ 	value = eel_preferences_get_enum (item->details->preference_name);
 
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
@@ -503,7 +504,8 @@ preferences_item_update_editable_string (EelPreferencesItem *item)
 	char *current_value;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_EDITABLE_STRING);
+	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_EDITABLE_STRING ||
+		          item->details->item_type == EEL_PREFERENCE_ITEM_EDITABLE_PASSWORD_STRING);
 
 	current_value = eel_preferences_get (item->details->preference_name);
 
@@ -518,7 +520,7 @@ preferences_item_update_editable_string (EelPreferencesItem *item)
 }
 
 static void
-preferences_item_create_editable_string (EelPreferencesItem *item)
+preferences_item_create_editable_string (EelPreferencesItem *item, gboolean password_string)
 {
 	GtkWidget *child;
 	
@@ -536,6 +538,10 @@ preferences_item_create_editable_string (EelPreferencesItem *item)
  	}
 	
 	eel_caption_set_title_label (EEL_CAPTION (child), "");
+
+	if (password_string) {
+		eel_text_caption_set_visibility (EEL_TEXT_CAPTION (child), FALSE);
+	}
 	
 	preferences_item_add_connection_child (item,
 					       child,
@@ -549,7 +555,8 @@ static void
 preferences_item_update_editable_integer (EelPreferencesItem *item)
 {
 	char *current_value;
-
+	char *tmp;
+	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_EDITABLE_INTEGER);
 
@@ -557,12 +564,12 @@ preferences_item_update_editable_integer (EelPreferencesItem *item)
 
 	g_assert (current_value != NULL);
 
-	if (strcmp (eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child)),
-		    current_value) != 0) {
+	tmp = eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child));
+	if (strcmp (tmp, current_value) != 0) {
 		eel_text_caption_set_text (EEL_TEXT_CAPTION (item->details->main_child), 
 					   current_value);
 	}
-
+	g_free (tmp);
 	g_free (current_value);
 }
 
@@ -595,14 +602,19 @@ preferences_item_update_enumeration_menu (EelPreferencesItem *item)
 	char *enumeration_id;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
-	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU);
+	g_return_if_fail (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU ||
+			  item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER);
 
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
-
+		
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
 	g_return_if_fail (eel_enumeration_id_get_length (enumeration_id) > 0);
 
-	current_value = eel_preferences_get_integer (item->details->preference_name);
+	if (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU) {
+		current_value = eel_preferences_get_enum (item->details->preference_name);
+	} else {
+		current_value = eel_preferences_get_integer (item->details->preference_name);
+	}
 
 	position = eel_enumeration_id_get_value_position (enumeration_id,
 							  current_value);
@@ -778,9 +790,13 @@ eel_preferences_item_new (const char *preference_name,
 	case EEL_PREFERENCE_ITEM_FONT:
 		preferences_item_create_font (item);
 		break;
-	
+
+	case EEL_PREFERENCE_ITEM_EDITABLE_PASSWORD_STRING:
+		preferences_item_create_editable_string (item, TRUE);
+		break;
+		
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
-		preferences_item_create_editable_string (item);
+		preferences_item_create_editable_string (item, FALSE);
 		break;	
 
 	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
@@ -788,6 +804,7 @@ eel_preferences_item_new (const char *preference_name,
 		break;	
 
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
 		preferences_item_create_enumeration_menu (item);
 		break;	
 
@@ -855,8 +872,8 @@ enumeration_radio_changed_callback (EelRadioButtonGroup *radio_button_group,
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
 	g_return_if_fail ((guint)i < eel_enumeration_id_get_length (enumeration_id));
 	
-	eel_preferences_set_integer (item->details->preference_name,
-				     eel_enumeration_id_get_nth_value (enumeration_id, i));
+	eel_preferences_set_enum (item->details->preference_name,
+				  eel_enumeration_id_get_nth_value (enumeration_id, i));
 	g_free (enumeration_id);
 }
 
@@ -927,15 +944,19 @@ enumeration_menu_changed_callback (EelStringPicker *string_picker,
  	selected_label = eel_string_picker_get_selected_string (string_picker);
 	g_return_if_fail (selected_label != NULL);
 
-	position = eel_enumeration_id_get_description_position (enumeration_id,
-								selected_label);
+	position = eel_string_picker_get_index_for_string (string_picker,
+							   selected_label);
 	g_free (selected_label);
 	g_return_if_fail (position != EEL_STRING_LIST_NOT_FOUND);
 	
 	new_value = eel_enumeration_id_get_nth_value (enumeration_id,
 						      position);
-	
-	eel_preferences_set_integer (item->details->preference_name, new_value);
+
+	if (item->details->item_type == EEL_PREFERENCE_ITEM_ENUMERATION_MENU) {
+		eel_preferences_set_enum (item->details->preference_name, new_value);
+	} else {
+		eel_preferences_set_integer (item->details->preference_name, new_value);
+	}
 
  	g_free (enumeration_id);
 }
@@ -969,8 +990,8 @@ enumeration_list_changed_callback (EelStringPicker *string_picker,
 		selected_label = eel_string_picker_get_selected_string (EEL_STRING_PICKER (connection->widget));
 		g_return_if_fail (selected_label != NULL);
 
-		position = eel_enumeration_id_get_description_position (enumeration_id,
-									selected_label);
+		position = eel_string_picker_get_index_for_string (EEL_STRING_PICKER (connection->widget),
+								   selected_label);
 		g_free (selected_label);
 		g_return_if_fail (position != EEL_STRING_LIST_NOT_FOUND);
 		
@@ -1045,7 +1066,8 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 	case EEL_PREFERENCE_ITEM_FONT:
 		preferences_item_update_font (item);
 		break;
-	
+
+	case EEL_PREFERENCE_ITEM_EDITABLE_PASSWORD_STRING:
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 		preferences_item_update_editable_string (item);
 		break;	
@@ -1055,6 +1077,7 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 		break;
 
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
 		preferences_item_update_enumeration_menu (item);
 		break;
 
@@ -1304,8 +1327,8 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 	case EEL_PREFERENCE_ITEM_BOOLEAN:
 		g_assert (GTK_IS_CHECK_BUTTON (item->details->main_child));
 		g_assert (GTK_IS_LABEL (GTK_BIN (item->details->main_child)->child));
-		gtk_label_set_text (GTK_LABEL (GTK_BIN (item->details->main_child)->child),
-				    description);
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (GTK_BIN (item->details->main_child)->child),
+						  description);
 		break;
 	case EEL_PREFERENCE_ITEM_ENUMERATION_VERTICAL_RADIO:
 	case EEL_PREFERENCE_ITEM_ENUMERATION_HORIZONTAL_RADIO:
@@ -1318,13 +1341,15 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 		g_assert (node != NULL);
 		g_assert (node->data != NULL);
 		g_assert (GTK_IS_LABEL (((GtkBoxChild *) node->data)->widget));
-		gtk_label_set_text (GTK_LABEL (((GtkBoxChild *) node->data)->widget),
-				    description);
+		gtk_label_set_text_with_mnemonic (GTK_LABEL (((GtkBoxChild *) node->data)->widget),
+						  description);
 		break;
 	case EEL_PREFERENCE_ITEM_FONT:
+	case EEL_PREFERENCE_ITEM_EDITABLE_PASSWORD_STRING:
 	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
 	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
 	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
 		g_assert (EEL_IS_CAPTION (item->details->main_child));
 		eel_caption_set_title_label (EEL_CAPTION (item->details->main_child),
 					     description);
@@ -1343,6 +1368,67 @@ eel_preferences_item_set_description (EelPreferencesItem *item,
 	}
 }
 
+/* Set the accessible description of a preferences item */
+void
+eel_preferences_item_set_accessible_description (EelPreferencesItem *item,
+						 const char *description)
+{
+	const char *descriptions[1];
+	
+	descriptions[0] = description;
+	
+	eel_preferences_item_set_accessible_description_array 
+		(item,
+		 1,
+		 descriptions);
+}
+
+/* Set the accessible description of a preferences item, taking an array
+ * of strings.  This is useful for radio enumerations that have multiple
+ * child widgets */
+void
+eel_preferences_item_set_accessible_description_array (EelPreferencesItem *item,
+						       int n_descriptions,
+						       const char **descriptions)
+{
+	int i;
+
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
+	g_return_if_fail (descriptions != NULL);
+
+	switch (item->details->item_type) {
+	case EEL_PREFERENCE_ITEM_BOOLEAN:
+	case EEL_PREFERENCE_ITEM_FONT:
+	case EEL_PREFERENCE_ITEM_EDITABLE_PASSWORD_STRING:
+	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
+	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU_INTEGER:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_LIST_VERTICAL:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_LIST_HORIZONTAL:
+		g_return_if_fail (n_descriptions != 1);
+		eel_accessibility_set_description (item->details->main_child,
+						   descriptions[0]);
+		break;
+	case EEL_PREFERENCE_ITEM_ENUMERATION_VERTICAL_RADIO:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_HORIZONTAL_RADIO:
+		for (i = 0; i < n_descriptions; i++) {
+			eel_radio_button_group_set_entry_accessible_description 
+				(EEL_RADIO_BUTTON_GROUP (item->details->main_child), 
+				 i,
+				 descriptions[i]);
+		}
+		break;
+	case EEL_PREFERENCE_ITEM_PADDING:
+		/* Padding doesnt have an accessible description */
+		break;
+	case EEL_PREFERENCE_ITEM_CUSTOM :
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
+
 static void
 eel_preferences_item_class_init (EelPreferencesItemClass *class)
 {
