@@ -8,27 +8,27 @@ typedef struct
 } Item;
 
 static Item items[] = {
-	{ "/gnome/GNOME2/share/pixmaps/nautilus/tahoe/theme_preview.png",
+	{ DATADIR "/pixmaps/nautilus/tahoe/theme_preview.png",
 	  "Tahoe",
 	  "This theme uses photo-realistic folders."
 	},
-	{ "/gnome/GNOME2/share/pixmaps/nautilus/crux_teal/theme_preview.png",
+	{ DATADIR "/pixmaps/nautilus/crux_teal/theme_preview.png",
 	  "Crux-Teal",
 	  "A Teal variation of the Crux theme."
 	},
-	{ "/gnome/GNOME2/share/pixmaps/nautilus/crux_eggplant/theme_preview.png",
+	{ DATADIR "/pixmaps/nautilus/crux_eggplant/theme_preview.png",
 	  "Crux-Eggplant",
 	  "An Eggplant variation of the Crux theme."
 	},
-	{ "/gnome/GNOME2/share/pixmaps/nautilus/gnome/theme_preview.png",
+	{ DATADIR "/pixmaps/nautilus/gnome/theme_preview.png",
 	  "GNOME",
 	  "This theme uses the classic GNOME icons."
 	},
-	{ "/gnome/GNOME2/share/pixmaps/nautilus/sierra/theme_preview.png",
+	{ DATADIR "/pixmaps/nautilus/sierra/theme_preview.png",
 	  "Sierra",
 	  "Uses manila folders and gray-green backgrounds."
 	},
-	{ "/gnome/GNOME2/share/pixmaps/nautilus/theme_preview.png",
+	{ DATADIR "/pixmaps/nautilus/theme_preview.png",
 	  "Default",
 	  "This is the default theme for Nautilus."
 	}
